package org.artifactory.projects;

import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;

/**
 * @author BarakH
 */
public class ProjectsUtilsTest {

    @Test(dataProvider = "provideProjectNames")
    public void testIsDefaultProject(String project, boolean isDefault) {
        boolean result = ProjectsUtils.isDefaultProject(project);
        Assert.assertEquals(result, isDefault);
    }

    @Test(dataProvider = "provideProjectNameAndExpectError")
    public void testAssertVirtualRepositoriesAreInProject(String projectName, String expectError) {
        ProjectsService projectsService = Mockito.mock(ProjectsService.class);
        Mockito.when(projectsService.getProjectRepositories("project1", false)).thenReturn(Set.of("A", "B"));
        Mockito.when(projectsService.getProjectRepositories("project2", false)).thenReturn(Set.of("C"));
        String error = "";
        try {
            ProjectsUtils.assertVirtualRepositoriesAreInProject(projectsService, projectName, List.of("A"));
        } catch (IllegalArgumentException e) {
            error = e.getMessage();
        }
        Assert.assertEquals(error, expectError);
    }

    @DataProvider
    public static Object[][] provideProjectNames() {
        return new Object[][] {
                {null, true},
                {"", true},
                {ProjectsService.DEFAULT_PROJECT, true},
                {"DEFAULT", true},
                {"project", false},
                {"default-project", false},
        };
    }

    @DataProvider
    public static Object[][] provideProjectNameAndExpectError() {
        return new Object[][] {
                {null, ""},
                {"", ""},
                {ProjectsService.DEFAULT_PROJECT, ""},
                {"project1", ""},
                {"project2", "Virtual repository must include only repositories from the same project it was created " +
                        "with"},
        };
    }
}
