/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.util;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import org.artifactory.aol.AolService;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.repo.DockerRepositoryAction;
import org.artifactory.common.ConstantValues;
import org.artifactory.rest.exception.ForbiddenWebAppException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Lior Azar
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class AolUtils {
    private static final Logger log = LoggerFactory.getLogger(AolUtils.class);

    public static void assertNotAol(String functionName) {
        if (isAol()) {
            log.warn("{} is not supported when running on the cloud", functionName);
            throw new ForbiddenWebAppException("Function is not supported when running on the cloud");
        }
    }

    public static void sendDockerRepoEvent(String repoKey, String version, DockerRepositoryAction action) {
        if (isAol() && !ConstantValues.dev.getBoolean()) {
            ContextHelper.get().beanForType(AolService.class).sendDockerRepoEvent(repoKey, version, action);
        }
    }

    public static boolean isAol() {
        return ContextHelper.get().beanForType(AolService.class).isAol();
    }

    public static boolean skipDockerDns() {
        return ContextHelper.get().beanForType(AolService.class).skipDockerDns();
    }

}
