package org.artifactory.storage.db.version.v224;

import org.artifactory.storage.db.version.converter.DBConverter;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;

/**
 * @author BarakH
 */
public class V224OptionalDeleteTableDBConverter implements DBConverter {

    private static final Logger log = LoggerFactory.getLogger(V224OptionalDeleteTableDBConverter.class);
    final static List<String> TABLES_TO_DROP = Arrays.asList(
            "aces", "acls", "permission_target_repos", "permission_targets", "users_groups", "groups", "user_props",
            "users");

    @Override
    public void convert(JdbcHelper jdbcHelper, DbType dbType) {
        log.debug("Starting schema conversion: Dropping deprecated tables");
        TABLES_TO_DROP.forEach(tableName -> dropTableIfExists(jdbcHelper, dbType, tableName));
        log.debug("Finished schema conversion: Dropping deprecated tables");
    }

    public void dropTableIfExists(JdbcHelper jdbcHelper, DbType dbType, String tableName) {
        String normalizedTableName = normalizeTableName(dbType, tableName);
        try {
            if (DbUtils.dropTableIfExists(jdbcHelper, dbType, normalizedTableName)) {
                log.info("Successfully dropped {} table", tableName);
            }
        } catch (SQLException e) {
            log.error("Failed to drop table {} with an error: {}", tableName, e.getMessage());
            log.debug(e.getMessage(), e);
        }
    }

    private String normalizeTableName(DbType dbType, String tableName) {
        if (dbType.equals(DbType.MYSQL)) {
            tableName = "`" + tableName + "`";
        }
        return tableName;
    }
}
