/*
 *
 * Copyright 2018 JFrog Ltd. All rights reserved.
 * JFROG PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package org.artifactory.storage.db.initializer;

import org.jfrog.storage.DbType;

/**
 * @author haims
 */
public class TestDbInitializerRegistry {

    public static TestDBInitializer getDbInitializer(String name, boolean forceAnotherDbForLocking) {
        String nameByType = name;
        nameByType = resolveDbTypeName(name, nameByType);
        try {
            DbType dbType = DbType.parse(nameByType.toUpperCase());
            TestDBInitializer dbInitializer = getDbInitializer(dbType);
            dbInitializer.setForceAnotherDbForLocking(forceAnotherDbForLocking);
            dbInitializer.setDbName(name);
            return dbInitializer;
        } catch (Exception e) {
            return new BaseTestDBInitializer(name);
        }
    }

    //TODO [by shayb]: we should not based on a string from the file name, but we should pass the real type from the
    //db.props to this method
    private static String resolveDbTypeName(String name, String nameByType) {
        if (name.endsWith("-docker")) {
            nameByType = name.replace("-docker", "");
        } else if (name.endsWith("-windows-1")) {
            nameByType = name.replace("-windows-1", "");
        }
        return nameByType;
    }

    private static TestDBInitializer getDbInitializer(DbType dbType) {
        switch (dbType) {
            case DERBY:
                return new DerbyTestDBInitializer();
            case MARIADB:
            case MYSQL:
                return new MySqlTestDBInitializer();
            case ORACLE:
                return new OracleTestDBInitializer();
            case MSSQL:
                return new MsSqlTestDBInitializer();
            case POSTGRESQL:
                return new PostgresqlTestDBInitializer();
            default:
                throw new RuntimeException("Could not identify DB type");
        }
    }
}
